@@ -100,7 +100,7 @@ opendso(char *dso, char *init, int domain)
 		    dlclose(handle);
 		}
 		if (dispatch.comm.pmapi_version != PMAPI_VERSION_2 &&
-		    dispatch.comm.pmapi_version != PMAPI_VERSION_3) {
+		    dispatch.comm.pmapi_version != PMAPI_VERSION_4) {
 		    printf("Error: Unsupported PMAPI version %d returned by DSO \"%s\"\n",
 			   dispatch.comm.pmapi_version, dso);
 		    dispatch.status = -1;
@@ -204,7 +204,7 @@ dodso(int pdu)
     int			length;
     pmDesc		desc;
     pmDesc		*desc_list = NULL;
-    pmResult		*result;
+    pmResult_v2		*result;
     pmLabelSet		*labelset = NULL;
     pmInResult		*inresult;
     int			i;
@@ -269,12 +269,12 @@ dodso(int pdu)
 		    if (desc_list)
 		        _dbDumpResult(stdout, result, desc_list);
                     else
-		        __pmDumpResult(stdout, result);
+		        __pmDumpResult_v2(stdout, result);
 		    /*
 		     * DSO PMDA will manage the pmResult skelton, but
 		     * we need to free the pmValueSets and values here
 		     */
-		    __pmFreeResultValues(result);
+		    __pmFreeResultValues_v2(result);
                 }
 		else {
 		    printf("Error: DSO fetch() failed: %s\n", pmErrStr(sts));
@@ -334,7 +334,7 @@ dodso(int pdu)
 	 
 	    sts = fillValues(result->vset[0], desc.type);
 	    if (sts < 0) {
-		pmFreeResult(result);
+		pmFreeResult_v2(result);
 		return;
 	    }
 
@@ -345,7 +345,7 @@ dodso(int pdu)
 	     * DSO PMDA will manage the pmResult skelton, but
 	     * we need to free the pmValueSets and values here
 	     */
-	    __pmFreeResultValues(result);
+	    __pmFreeResultValues_v2(result);
 
 	    break;
 
