@@ -88,8 +88,7 @@ build_dsotab(void)
 	return PM_ERR_GENERIC;
     if ((config = pmGetOptionalConfig("PCP_PMCDCONF_PATH")) == NULL)
 	return PM_ERR_GENERIC;
-    strncpy(configFileName, config, sizeof(configFileName));
-    configFileName[sizeof(configFileName) - 1] = '\0';
+    pmstrncpy(configFileName, sizeof(configFileName), config);
     if (pmDebugOptions.context) {
 	fprintf(stderr, "build_dsotab: parsing %s\n", configFileName);
     }
@@ -125,6 +124,19 @@ build_dsotab(void)
 	     */
 	    goto eatline;
 	}
+	if (strncmp(p, "proc", 4) == 0) {
+	    /*
+	     * the proc PMDA is an exception now too ... we run it as root
+	     * (daemon) but we still want to make the DSO available for any
+	     * local context users.  We add this explicitly below.
+	     */
+	    domain = 3;
+	    init = "proc_init";
+	    pmsprintf(pathbuf, sizeof(pathbuf), "%s/proc/pmda_proc.so", pmdas);
+	    name = pathbuf;
+	    peekc = *p;
+	    goto dsoload;
+	}
 	if (strncmp(p, "linux", 5) == 0) {
 	    /*
 	     * the Linux PMDA is an exception now too ... we run it as root
@@ -260,6 +272,21 @@ __pmLookupDSO(int domain)
     return NULL;
 }
 
+void
+__pmClearDSOProfile(int handle)
+{
+    int		i;
+    for (i = 0; i < numdso; i++) {
+	if (dsotab[i].ctx_last_prof == handle) {
+	    if (pmDebugOptions.profile) {
+		fprintf(stderr, "__pmClearDSOProfile(%d) -> dsotab[%d], domain=%d\n", handle, i, dsotab[i].domain);
+	    }
+	    dsotab[i].ctx_last_prof = -1;
+	}
+    }
+    return;
+}
+
 static void
 EndLocalContext(void)
 {
@@ -417,11 +444,11 @@ __pmConnectLocal(__pmHashCtl *attrs)
 	/*
 	 * the PMDA interface / PMAPI version discovery as a "challenge" ...
 	 * for pmda_interface it is all the bits being set,
-	 * for pmapi_version it is the complement of the one you are using now
+	 * for pmapi_version it is the complement of the oldest supported.
 	 */
 	challenge = 0xff;
 	dp->dispatch.comm.pmda_interface = challenge;
-	dp->dispatch.comm.pmapi_version = (~PMAPI_VERSION) & 0xff;
+	dp->dispatch.comm.pmapi_version = PMAPI_VERSION_2 & 0xff;
 	dp->dispatch.comm.flags = 0;
 	dp->dispatch.status = 0;
 
@@ -448,7 +475,8 @@ connected:
 		dlclose(dp->handle);
 		dp->domain = -1;
 	    }
-	    else if (dp->dispatch.comm.pmapi_version != PMAPI_VERSION_2) {
+	    else if (dp->dispatch.comm.pmapi_version != PMAPI_VERSION_2 &&
+		     dp->dispatch.comm.pmapi_version != PMAPI_VERSION_4) {
 		pmprintf("__pmConnectLocal: Error: Unknown PMAPI version %d "
 			 "in \"%s\" DSO\n",
 			 dp->dispatch.comm.pmapi_version, dp->name);
@@ -534,6 +562,7 @@ __pmLocalPMDA(int op, int domain, const char *name, const char *init)
 		}
 	    }
 	    dsotab[numdso].handle = NULL;
+	    dsotab[numdso].ctx_last_prof = -1;
 	    numdso++;
 	    break;
 
@@ -610,7 +639,6 @@ pmSpecLocalPMDA(const char *spec)
     char	*ap;
 
     if ((arg = sbuf = strdup(spec)) == NULL) {
-	sts = -oserror();
 	pmNoMem("pmSpecLocalPMDA dup spec", strlen(spec)+1, PM_RECOV_ERR);
 	return "strdup failed";
     }
