@@ -179,6 +179,9 @@ enum {
     _PM_NETSTAT_TCPEXT_TCPAOKEYNOTFOUND,
     _PM_NETSTAT_TCPEXT_TCPAOGOOD,
     _PM_NETSTAT_TCPEXT_TCPAODROPPEDICMPS,
+    _PM_NETSTAT_TCPEXT_PAWSOLDACK,
+    _PM_NETSTAT_TCPEXT_TSECRREJECTED,
+    _PM_NETSTAT_TCPEXT_PAWSTIMEWAIT,
 
     _PM_NETSTAT_TCPEXT_NFIELDS /* must be last */
 };
