@@ -77,7 +77,7 @@ _cleanup()
     lockfile=`cat $tmp/lock 2>/dev/null`
     [ -n "$lockfile" ] && rm -f "$lockfile"
     rm -rf $tmp
-    $VERY_VERBOSE && echo "End: `date '+%F %T.%N'`"
+    $VERY_VERBOSE && echo "End [janitor]: `date '+%F %T.%N'`"
 }
 
 trap "_cleanup; exit \$status" 0 1 2 3 15
@@ -95,22 +95,10 @@ CONTROLDIR=$PCP_PMLOGGERCONTROL_PATH.d
 # 
 COMPRESS=""
 COMPRESS_CMDLINE=""
-if which xz >/dev/null 2>&1
-then
-    if xz -0 --block-size=10MiB </dev/null >/dev/null 2>&1
-    then
-	# want minimal overheads, -0 is the same as --fast
-	COMPRESS_DEFAULT="xz -0 --block-size=10MiB"
-    else
-	COMPRESS_DEFAULT=xz
-    fi
-else
-    # overridden by $PCP_COMPRESS or if not set, no compression
-    COMPRESS_DEFAULT=""
-fi
+COMPRESS_DEFAULT="pmlogcompress"
 COMPRESSREGEX=""
 COMPRESSREGEX_CMDLINE=""
-COMPRESSREGEX_DEFAULT="\.(index|Z|gz|bz2|zip|xz|lzma|lzo|lz4)$"
+COMPRESSREGEX_DEFAULT="\.(index|Z|gz|bz2|zip|xz|lzma|lzo|lz4|zst)$"
 
 # determine path for pwd command to override shell built-in
 PWDCMND=`which pwd 2>/dev/null | $PCP_AWK_PROG '
@@ -286,19 +274,6 @@ then
     exit
 fi
 
-_error()
-{
-    [ -n "$controlfile" ] && echo "$prog: [$controlfile:$line]"
-    echo "Error: $@"
-    touch $tmp/err
-}
-
-_warning()
-{
-    [ -n "$controlfile" ] && echo "$prog [$controlfile:$line]"
-    echo "Warning: $@"
-}
-
 _restarting()
 {
     $PCP_ECHO_PROG $PCP_ECHO_N "Restarting$iam pmlogger for host \"$host\" ...""$PCP_ECHO_C"
@@ -354,319 +329,27 @@ rm -f $tmp/err
 
 rm -f $tmp/pmloggers
 
-_parse_control()
+# come here from _parse_log_control() once per valid line in a control
+# file ... see utilproc.sh for interface definitions
+#
+_callback_log_control()
 {
-    controlfile="$1"
-    line=0
-
-    if echo "$controlfile" | grep -q -e '\.rpmsave$' -e '\.rpmnew$' -e '\.rpmorig$' \
-	-e '\.dpkg-dist$' -e '\.dpkg-old$' -e '\.dpkg-new$' >/dev/null 2>&1
+    if $VERY_VERBOSE
     then
-	_warning "ignored backup control file \"$controlfile\""
-	return
+	pflag=''
+	[ $primary = y ] && pflag=' -P'
+	echo "Checking for: pmlogger$pflag -h $host ... in $dir ..."
     fi
 
-    sed \
-	-e "s;PCP_ARCHIVE_DIR;$PCP_ARCHIVE_DIR;g" \
-	-e "s;PCP_LOG_DIR;$PCP_LOG_DIR;g" \
-	$controlfile | \
-    while read host primary socks dir args
-    do
-	# start in one place for each iteration (beware relative paths)
-	cd "$here"
-	line=`expr $line + 1`
+    pid=`_find_matching_pmlogger`
 
-
-	if $VERY_VERBOSE 
-	then
-	    case "$host"
-	    in
-	    \#*|'')	# comment or empty
-			;;
-	    *)		echo "[$controlfile:$line] host=\"$host\" primary=\"$primary\" socks=\"$socks\" dir=\"$dir\" args=\"$args\""
-	    		;;
-	    esac
-	fi
-
-	case "$host"
-	in
-	    \#*|'')	# comment or empty
-		continue
-		;;
-	    \$*)	# in-line variable assignment
-		cmd=`echo "$host $primary $socks $dir $args" \
-		     | sed -n \
-			 -e "/='/s/\(='[^']*'\).*/\1/" \
-			 -e '/="/s/\(="[^"]*"\).*/\1/' \
-			 -e '/=[^"'"'"']/s/[;&<>|].*$//' \
-			 -e '/^\\$[A-Za-z][A-Za-z0-9_]*=/{
-s/^\\$//
-s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
-}'`
-		if [ -z "$cmd" ]
-		then
-		    # in-line command, not a variable assignment
-		    _warning "in-line command is not a variable assignment, line ignored"
-		else
-		    case "$cmd"
-		    in
-			'export PATH;'*)
-			    _warning "cannot change \$PATH, line ignored"
-			    ;;
-			'export IFS;'*)
-			    _warning "cannot change \$IFS, line ignored"
-			    ;;
-			*)
-			    echo eval $cmd >>$tmp/cmd
-			    eval $cmd
-			    ;;
-		    esac
-		fi
-		continue
-		;;
-	esac
-
-	# set the version and other variables
-	#
-	[ -f $tmp/cmd ] && . $tmp/cmd
-
-	if [ -z "$version" -o "$version" = "1.0" ]
-	then
-	    if [ -z "$version" ]
-	    then
-		_warning "processing default version 1.0 control format"
-		version=1.0
-	    fi
-	    args="$dir $args"
-	    dir="$socks"
-	    socks=n
-	fi
-
-	# do shell expansion of $dir if needed
-	#
-	_do_dir_and_args
-
-	if [ -z "$primary" -o -z "$socks" -o -z "$dir" -o -z "$args" ]
-	then
-	    _error "insufficient fields in control file record"
-	    continue
-	fi
-
-	# substitute LOCALHOSTNAME marker in this config line
-	# (differently for directory and pcp -h HOST arguments)
-	#
-	dirhostname=`hostname || echo localhost`
-	dir=`echo $dir | sed -e "s;LOCALHOSTNAME;$dirhostname;"`
-	[ $primary = y -o "x$host" = xLOCALHOSTNAME ] && host=local:
-
-	if $VERY_VERBOSE
-	then
-	    pflag=''
-	    [ $primary = y ] && pflag=' -P'
-	    echo "Checking for: pmlogger$pflag -h $host ... in $dir ..."
-	fi
-
-	# check for directory duplicate entries
-	#
-	if [ "`grep $dir $tmp/dir`" = "$dir" ]
-	then
-	    _error "Cannot start more than one pmlogger instance for archive directory \"$dir\""
-	    continue
-	else
-	    echo "$dir" >>$tmp/dir
-	fi
-
-	# make sure output directory hierarchy exists and $PCP_USER
-	# user can write there
-	#
-	if [ ! -d "$dir" ]
-	then
-	    # mode rwxrwxr-x is the default for pcp:pcp dirs
-	    umask 002
-	    mkdir -p -m 0775 "$dir" >$tmp/tmp 2>&1
-	    # reset the default mode to rw-rw-r- for files
-	    umask 022
-	    if [ ! -d "$dir" ]
-	    then
-		cat $tmp/tmp
-		_error "cannot create directory ($dir) for PCP archive files"
-		continue
-	    else
-		_warning "creating directory ($dir) for PCP archive files"
-	    fi
-	fi
-
-	if cd "$dir"
-	then
-	    :
-	else
-	    _error "cannot chdir to directory ($dir) for PCP archive files"
-	    continue
-	fi
-	dir=`$PWDCMND`
-
-	if [ ! -w "$dir" ]
-	then
-	    _warning "no write access in $dir, skip lock file processing"
-	    ls -ld "$dir"
-	else
-	    # demand mutual exclusion
-	    #
-	    rm -f $tmp/stamp $tmp/out
-	    delay=200	# tenths of a second
-	    while [ $delay -gt 0 ]
-	    do
-		if pmlock -v "$dir/lock" >$tmp/out 2>&1
-		then
-		    echo "$dir/lock" >$tmp/lock
-		    if $VERY_VERBOSE
-		    then
-			echo "Acquired lock:"
-			ls -l $dir/lock
-		    fi
-		    break
-		else
-		    [ -f $tmp/stamp ] || touch -t `pmdate -30M %Y%m%d%H%M` $tmp/stamp
-		    find $tmp/stamp -newer "$dir/lock" -print 2>/dev/null >$tmp/tmp
-		    if [ -s $tmp/tmp ]
-		    then
-			if [ -f "$dir/lock" ]
-			then
-			    echo "$prog: Warning: removing lock file older than 30 minutes"
-			    LC_TIME=POSIX ls -l $dir/lock
-			    rm -f "$dir/lock"
-			else
-			    # there is a small timing window here where pmlock
-			    # might fail, but the lock file has been removed by
-			    # the time we get here, so just keep trying
-			    #
-			    :
-			fi
-		    fi
-		fi
-		pmsleep 0.1
-		delay=`expr $delay - 1`
-	    done
-
-	    if [ $delay -eq 0 ]
-	    then
-		# failed to gain mutex lock
-		#
-		# maybe pmlogger_daily is running ... check it, and silently
-		# move on if this is the case
-		#
-		# Note: $PCP_RUN_DIR may not exist (see pmlogger_daily note),
-		#       but only if pmlogger_daily has not run, so no chance
-		#       of a collision
-		#
-		if [ -f "$PCP_RUN_DIR"/pmlogger_daily.pid ]
-		then
-		    # maybe, check pid matches a running /bin/sh
-		    #
-		    pid=`cat "$PCP_RUN_DIR"/pmlogger_daily.pid`
-		    if _get_pids_by_name sh | grep "^$pid\$" >/dev/null
-		    then
-			# seems to be still running ... nothing for us to see
-			# or do here
-			#
-			continue
-		    fi
-		fi
-		if [ -f "$dir/lock" ]
-		then
-		    echo "$prog: Warning: is another PCP cron job running concurrently?"
-		    LC_TIME=POSIX ls -l $dir/lock
-		else
-		    echo "$prog: `cat $tmp/out`"
-		fi
-		_warning "failed to acquire exclusive lock ($dir/lock) ..."
-		continue
-	    fi
-	fi
-
-	pid=''
-	if [ "$primary" = y ]
-	then
-	    if test -e "$PCP_TMP_DIR/pmlogger/primary"
-	    then
-		if $VERY_VERBOSE
-		then 
-		    _host=`sed -n 2p <"$PCP_TMP_DIR/pmlogger/primary"`
-		    _arch=`sed -n 3p <"$PCP_TMP_DIR/pmlogger/primary"`
-		    echo "... try $PCP_TMP_DIR/pmlogger/primary: host=$_host arch=$_arch"
-		fi
-		pid=`_get_primary_logger_pid`
-		if [ -z "$pid" ]
-		then
-		    if $VERY_VERBOSE
-		    then
-			echo "primary pmlogger process pid not found"
-			ls -l "$PCP_RUN_DIR/pmlogger.pid"
-			ls -l "$PCP_TMP_DIR/pmlogger"
-		    fi
-		elif _get_pids_by_name pmlogger | grep "^$pid\$" >/dev/null
-		then
-		    $VERY_VERBOSE && echo "primary pmlogger process $pid identified, OK"
-		    $VERY_VERY_VERBOSE && $PCP_PS_PROG $PCP_PS_ALL_FLAGS | grep -E '[P]ID|/[p]mlogger( |$)'
-		else
-		    $VERY_VERBOSE && echo "primary pmlogger process $pid not running"
-		    $VERY_VERY_VERBOSE && $PCP_PS_PROG $PCP_PS_ALL_FLAGS | grep -E '[P]ID|/[p]mlogger( |$)'
-		    pid=''
-		fi
-	    else
-		if $VERY_VERBOSE
-		then
-		    echo "$PCP_TMP_DIR/pmlogger/primary: missing?"
-		    echo "Contents of $PCP_TMP_DIR/pmlogger"
-		    ls -l $PCP_TMP_DIR/pmlogger
-		    echo "--- end of ls output ---"
-		fi
-	    fi
-	else
-	    for log in $PCP_TMP_DIR/pmlogger/[0-9]*
-	    do
-		[ "$log" = "$PCP_TMP_DIR/pmlogger/[0-9]*" ] && continue
-		if $VERY_VERBOSE
-		then
-		    _host=`sed -n 2p <$log`
-		    _arch=`sed -n 3p <$log`
-		    $PCP_ECHO_PROG $PCP_ECHO_N "... try $log host=$_host arch=$_arch: ""$PCP_ECHO_C"
-		fi
-		# throw away stderr in case $log has been removed by now
-		match=`sed -e '3s@/[^/]*$@@' $log 2>/dev/null | \
-		$PCP_AWK_PROG '
-BEGIN				{ m = 0 }
-NR == 3 && $0 == "'$dir'"	{ m = 2; next }
-END				{ print m }'`
-		$VERY_VERBOSE && echo "match=$match"
-		if [ "$match" = 2 ]
-		then
-		    pid=`echo $log | sed -e 's,.*/,,'`
-		    if _get_pids_by_name pmlogger | grep "^$pid\$" >/dev/null
-		    then
-			$VERY_VERBOSE && echo "pmlogger process $pid identified, OK"
-			$VERY_VERY_VERBOSE && $PCP_PS_PROG $PCP_PS_ALL_FLAGS | grep -E '[P]ID|/[p]mlogger( |$)'
-			break
-		    fi
-		    $VERY_VERBOSE && echo "pmlogger process $pid not running, skip"
-		    $VERY_VERY_VERBOSE && $PCP_PS_PROG $PCP_PS_ALL_FLAGS | grep -E '[P]ID|/[p]mlogger( |$)'
-		    pid=''
-		else
-		    $VERY_VERBOSE && echo "different directory, skip"
-		fi
-	    done
-	fi
-
-	if [ -n "$pid" ]
-	then
-	    # found matching pmlogger ... cull this one from
-	    $VERY_VERBOSE && echo "[$controlfile:$line] match PID $pid, nothing to be done"
-	    sed <$tmp/loggers >$tmp/tmp -e "/^$pid	/d"
-	    mv $tmp/tmp $tmp/loggers
-	fi
-
-	_unlock "$dir"
-    done
+    if [ -n "$pid" ]
+    then
+	# found matching pmlogger ... cull this one from
+	$VERY_VERBOSE && echo "[$filename:$line] match PID $pid, nothing to be done"
+	sed <$tmp/loggers >$tmp/tmp -e "/^$pid	/d"
+	mv $tmp/tmp $tmp/loggers
+    fi
 }
 
 # Pass 1 - look in the pmlogger status files for those with -m "note"
@@ -729,9 +412,39 @@ done
 append=`ls $CONTROLDIR 2>/dev/null | LC_COLLATE=POSIX sort | sed -e "s;^;$CONTROLDIR/;g"`
 for c in $CONTROL $append
 do
-    _parse_control "$c"
+    _parse_log_control "$c"
 done
 
+# check for any archives from remote pmloggers via pmproxy or
+# pmlogpush ... if found, synthesize a control file for them
+#
+here=`pwd`
+if cd "$PCP_REMOTE_ARCHIVE_DIR"
+then
+    for _host in *
+    do
+	# TODO - does this need to be smarter?  e.g. check for some
+	# minimal dir contents (.index file?)
+	if [ -d "$_host" ]
+	then
+	    $VERBOSE && echo "Info: processing archives from remote pmlogger on host $_host"
+	    echo '$version=1.1' >$tmp/control
+	    # optional global controls first
+	    [ -f "./control" ] && cat "./control" >>$tmp/control
+	    # optional per-host controls next
+	    [ -f "$_host/control" ] && cat "$_host/control" >>$tmp/control
+	    echo "$_host	n n PCP_REMOTE_ARCHIVE_DIR/$_host +" >>$tmp/control
+	    if $VERY_VERBOSE
+	    then
+		echo >&2 "Synthesized control file ..."
+		cat >&2 $tmp/control
+	    fi
+	    _parse_log_control $tmp/control
+	fi
+    done
+    cd $here
+fi
+
 if [ ! -s $tmp/loggers ]
 then
     # Take early exit if nothing to be done
@@ -742,7 +455,7 @@ fi
 # errors/warnings from here on have nothing to do with any specific
 # control file (or line in a control file)
 #
-controlfile=''
+filename=''
 
 # Pass 4 - kill of orphaned pmloggers
 #
